package cn.fw.hermes.sdk.v2.para;

import cn.fw.common.validator.EnumValue;
import cn.fw.hermes.sdk.v2.enums.MsgType;
import org.hibernate.validator.constraints.ScriptAssert;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Map;

/**
 * 消息体
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : 消息体
 * @date : 2023-07-22 09:58
 */
@ScriptAssert.List({
        @ScriptAssert(lang = "javascript", script = "_this.checkDataValue()", message = "消息数据不正确"),
})
public class MsgBody implements Serializable {
    private static final long serialVersionUID = 45480077093841L;
    /**
     * 消息类型
     *
     * @see cn.fw.hermes.sdk.v2.enums.MsgType
     */
    @NotNull(message = "消息类型不能为空")
    @EnumValue(enumClass = cn.fw.hermes.sdk.v2.enums.MsgType.class, valueMethod = "getCode", message = "消息类型不正确")
    private Integer msgType;
    /**
     * 文本信息
     */
    private String text;
    /**
     * 语音信息 语音地址 非空
     * 格式为：(语音地址, 时长(秒))
     */
    private HermesPair<String, Integer> voiceData;
    /**
     * 视频信息 视频地址非空
     * 格式为：(封面图, 视频地址, 时长(秒))
     */
    private HermesTriple<String, String, Integer> videoData;
    /**
     * 图片信息 原图地址 非空
     * 格式为：(缩略图,中图地址,原图地址)
     */
    private HermesTriple<String, String, String> imageData;
    /**
     * 地理位置信息 经度,纬度 非空
     * 格式为：(地址, 经度(lat),纬度(lng))
     */
    private HermesTriple<String, BigDecimal, BigDecimal> locationData;
    /**
     * 表情消息
     * 表情下标
     */
    private Integer emojiIdx;
    private Map<String, Object> customData;

    public Integer getMsgType() {
        return msgType;
    }

    public void setMsgType(Integer msgType) {
        this.msgType = msgType;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public HermesPair<String, Integer> getVoiceData() {
        return voiceData;
    }

    public void setVoiceData(HermesPair<String, Integer> voiceData) {
        this.voiceData = voiceData;
    }

    public HermesTriple<String, String, Integer> getVideoData() {
        return videoData;
    }

    public void setVideoData(HermesTriple<String, String, Integer> videoData) {
        this.videoData = videoData;
    }

    public HermesTriple<String, String, String> getImageData() {
        return imageData;
    }

    public void setImageData(HermesTriple<String, String, String> imageData) {
        this.imageData = imageData;
    }

    public HermesTriple<String, BigDecimal, BigDecimal> getLocationData() {
        return locationData;
    }

    public void setLocationData(HermesTriple<String, BigDecimal, BigDecimal> locationData) {
        this.locationData = locationData;
    }

    public Integer getEmojiIdx() {
        return emojiIdx;
    }

    public void setEmojiIdx(Integer emojiIdx) {
        this.emojiIdx = emojiIdx;
    }

    public Map<String, Object> getCustomData() {
        return customData;
    }

    public void setCustomData(Map<String, Object> customData) {
        this.customData = customData;
    }

    public boolean checkDataValue() {
        MsgType type = MsgType.ofCode(msgType);
        if (type == null) {
            return false;
        }
        switch (type) {
            case TEXT:
                return text != null && !text.isEmpty();
            case IMAGE:
                return imageData != null && imageData.right != null && !imageData.right.isEmpty();
            case VOICE:
                return videoData != null && voiceData.left != null && !voiceData.left.isEmpty() &&
                        voiceData.right != null && voiceData.right >= 0;
            case VIDEO:
                return videoData != null && videoData.middle != null && !videoData.middle.isEmpty() &&
                        videoData.right != null && videoData.right >= 0;
            case EMOJI:
                return emojiIdx != null && emojiIdx >= 0;
            case LOCATION:
                return locationData != null && locationData.middle != null &&
                        locationData.right != null;
            case CUSTOM:
                return customData != null && !customData.isEmpty();
            default:
                return false;
        }
    }

}
