package cn.fw.hermes.sdk.v2.para;

import cn.fw.hermes.sdk.v2.enums.AccountType;
import cn.fw.hermes.sdk.v2.enums.Platform;
import lombok.Data;
import org.hibernate.validator.constraints.ScriptAssert;

import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 消息数据
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : 消息数据
 * @date : 2023-07-22 09:45
 */
@ScriptAssert.List({
    @ScriptAssert(lang = "javascript", script = "_this.checkPlatform()", message = "平台类型不正确"),
    @ScriptAssert(lang = "javascript", script = "_this.checkUser()", message = "目标用户或者账号类型不正确"),
})
@Data
public class MsgV2Condition implements Serializable {
    private static final long serialVersionUID = 54918890077093841L;
    /**
     * 预留属性，当前版本不使用，后续支持
     * <p> 消息离线保存时长(秒)，最大支持7天。 </p>
     * <p> 在此时长内用户不在线时，会等待用户上线自动推送(只针对指定用户id的消息) </p>
     * <p> 不设置时用户不在线会直接返回错误 </p>
     */
    @Min(0)
    @Max(604_800)
    private Integer msgLife;
    /**
     * 目标对应userId
     */
    private Long userId;
    /**
     * **平台**
     * <p>
     *     ** 后续只会用第一个平台
     *     为了兼容之前版本数据类型不做改动
     *     为空时默认app平台
     * </p>
     * @slice 2.0.1
     *
     * @see cn.fw.hermes.sdk.v2.enums.Platform
     */
    private List<Integer> platforms;
    /**
     * 账号类型
     *
     * @see cn.fw.hermes.sdk.v2.enums.AccountType
     */
    private Integer userType;
    /**
     * 是否所有用户
     */
    private Boolean fullUser;
    /**
     * 消息体
     */
    @Valid
    @NotNull(message = "消息体不能为空")
    private MsgBody msgElement;
    /**
     * 需要结果通知
     */
    private Boolean resultNotice;

    /**
     * 创建文本消息
     *
     * @param text         文本消息
     * @param userId       用户id
     * @param cuser        是否是c端用户
     * @param msgLife      消息最大保留时长（预留）
     * @param fullUser     是否全部用户
     * @param resultNotice 是否需要结果通知
     * @param platforms    平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createTextCondition(String text,
                                                     Long userId,
                                                     boolean cuser,
                                                     Integer msgLife,
                                                     boolean fullUser,
                                                     boolean resultNotice,
                                                     Integer... platforms) {
        MsgV2Condition condition = createBaseCondition(userId, cuser, msgLife, fullUser, resultNotice, platforms);
        MsgBody body = new MsgBody();
        body.setMsgType(1);
        body.setText(text);
        condition.setMsgElement(body);
        return condition;
    }

    /**
     * 创建基于app端推送的文本消息
     *
     * @param text   文本消息
     * @param userId 用户id
     * @param cuser  是否是c端用户
     * @return 消息参数
     */
    public static MsgV2Condition createTextCondition(String text,
                                                     Long userId,
                                                     boolean cuser) {
        return createTextCondition(text, userId, cuser, 1);
    }

    /**
     * 创建文本消息
     *
     * @param text      文本消息
     * @param userId    用户id
     * @param cuser     是否是c端用户
     * @param platforms 平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createTextCondition(String text,
                                                     Long userId,
                                                     boolean cuser,
                                                     Integer... platforms) {
        return createTextCondition(text, userId, cuser, 604_800, false, true, platforms);
    }

    /**
     * 创建图片消息
     *
     * @param imageData    图片信息
     * @param userId       用户id
     * @param cuser        是否是c端用户
     * @param msgLife      消息最大保留时长（预留）
     * @param fullUser     是否全部用户
     * @param resultNotice 是否需要结果通知
     * @param platforms    平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createImageCondition(HermesTriple<String, String, String> imageData,
                                                      Long userId,
                                                      boolean cuser,
                                                      Integer msgLife,
                                                      boolean fullUser,
                                                      boolean resultNotice,
                                                      Integer... platforms) {

        MsgV2Condition condition = createBaseCondition(userId, cuser, msgLife, fullUser, resultNotice, platforms);
        MsgBody body = new MsgBody();
        body.setMsgType(2);
        body.setImageData(imageData);
        condition.setMsgElement(body);
        return condition;
    }

    /**
     * 创建图片消息
     *
     * @param imageData 图片信息
     * @param userId    用户id
     * @param cuser     是否是c端用户
     * @return 消息参数
     */
    public static MsgV2Condition createImageCondition(HermesTriple<String, String, String> imageData,
                                                      Long userId,
                                                      boolean cuser) {
        return createImageCondition(imageData, userId, cuser, 1);
    }

    /**
     * 创建图片消息
     *
     * @param imageData 图片信息
     * @param userId    用户id
     * @param cuser     是否是c端用户
     * @param platforms 平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createImageCondition(HermesTriple<String, String, String> imageData,
                                                      Long userId,
                                                      boolean cuser,
                                                      Integer... platforms) {
        return createImageCondition(imageData, userId, cuser, 604_800, false, true, platforms);
    }


    /**
     * 创建语音消息
     *
     * @param data         语音信息
     * @param userId       用户id
     * @param cuser        是否是c端用户
     * @param msgLife      消息最大保留时长（预留）
     * @param fullUser     是否全部用户
     * @param resultNotice 是否需要结果通知
     * @param platforms    平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createVoiceCondition(HermesPair<String, Integer> data,
                                                      Long userId,
                                                      boolean cuser,
                                                      Integer msgLife,
                                                      boolean fullUser,
                                                      boolean resultNotice,
                                                      Integer... platforms) {

        MsgV2Condition condition = createBaseCondition(userId, cuser, msgLife, fullUser, resultNotice, platforms);
        MsgBody body = new MsgBody();
        body.setMsgType(3);
        body.setVoiceData(data);
        condition.setMsgElement(body);
        return condition;
    }

    /**
     * 创建语音消息
     *
     * @param data   语音信息
     * @param userId 用户id
     * @param cuser  是否是c端用户
     * @return 消息参数
     */
    public static MsgV2Condition createVoiceCondition(HermesPair<String, Integer> data,
                                                      Long userId,
                                                      boolean cuser) {
        return createVoiceCondition(data, userId, cuser, 1);
    }


    /**
     * 创建语音消息
     *
     * @param data      语音信息
     * @param userId    用户id
     * @param cuser     是否是c端用户
     * @param platforms 平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createVoiceCondition(HermesPair<String, Integer> data,
                                                      Long userId,
                                                      boolean cuser,
                                                      Integer... platforms) {
        return createVoiceCondition(data, userId, cuser, 604_800, false, true, platforms);
    }


    /**
     * 创建视频消息
     *
     * @param data         视频信息
     * @param userId       用户id
     * @param cuser        是否是c端用户
     * @param msgLife      消息最大保留时长（预留）
     * @param fullUser     是否全部用户
     * @param resultNotice 是否需要结果通知
     * @param platforms    平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createVideoCondition(HermesTriple<String, String, Integer> data,
                                                      Long userId,
                                                      boolean cuser,
                                                      Integer msgLife,
                                                      boolean fullUser,
                                                      boolean resultNotice,
                                                      Integer... platforms) {

        MsgV2Condition condition = createBaseCondition(userId, cuser, msgLife, fullUser, resultNotice, platforms);
        MsgBody body = new MsgBody();
        body.setMsgType(4);
        body.setVideoData(data);
        condition.setMsgElement(body);
        return condition;
    }


    /**
     * 创建视频消息
     *
     * @param data   视频信息
     * @param userId 用户id
     * @param cuser  是否是c端用户
     * @return 消息参数
     */
    public static MsgV2Condition createVideoCondition(HermesTriple<String, String, Integer> data,
                                                      Long userId,
                                                      boolean cuser) {
        return createVideoCondition(data, userId, cuser, 1);
    }


    /**
     * 创建视频消息
     *
     * @param data      视频信息
     * @param userId    用户id
     * @param cuser     是否是c端用户
     * @param platforms 平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createVideoCondition(HermesTriple<String, String, Integer> data,
                                                      Long userId,
                                                      boolean cuser,
                                                      Integer... platforms) {
        return createVideoCondition(data, userId, cuser, 604_800, false, true, platforms);
    }

    /**
     * 创建emoji消息
     *
     * @param data         表情下标
     * @param userId       用户id
     * @param cuser        是否是c端用户
     * @param msgLife      消息最大保留时长（预留）
     * @param fullUser     是否全部用户
     * @param resultNotice 是否需要结果通知
     * @param platforms    平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createEmojiCondition(Integer data,
                                                      Long userId,
                                                      boolean cuser,
                                                      Integer msgLife,
                                                      boolean fullUser,
                                                      boolean resultNotice,
                                                      Integer... platforms) {

        MsgV2Condition condition = createBaseCondition(userId, cuser, msgLife, fullUser, resultNotice, platforms);
        MsgBody body = new MsgBody();
        body.setMsgType(5);
        body.setEmojiIdx(data);
        condition.setMsgElement(body);
        return condition;
    }

    /**
     * 创建emoji消息
     *
     * @param data   表情下标
     * @param userId 用户id
     * @param cuser  是否是c端用户
     * @return 消息参数
     */
    public static MsgV2Condition createEmojiCondition(Integer data,
                                                      Long userId,
                                                      boolean cuser) {
        return createEmojiCondition(data, userId, cuser, 1);
    }

    /**
     * 创建emoji消息
     *
     * @param data      表情下标
     * @param userId    用户id
     * @param cuser     是否是c端用户
     * @param platforms 平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createEmojiCondition(Integer data,
                                                      Long userId,
                                                      boolean cuser,
                                                      Integer... platforms) {
        return createEmojiCondition(data, userId, cuser, 604_800, false, true, platforms);
    }


    /**
     * 创建地理位置消息
     *
     * @param data         地理位置信息
     * @param userId       用户id
     * @param cuser        是否是c端用户
     * @param msgLife      消息最大保留时长（预留）
     * @param fullUser     是否全部用户
     * @param resultNotice 是否需要结果通知
     * @param platforms    平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createLocationCondition(HermesTriple<String, BigDecimal, BigDecimal> data,
                                                         Long userId,
                                                         boolean cuser,
                                                         Integer msgLife,
                                                         boolean fullUser,
                                                         boolean resultNotice,
                                                         Integer... platforms) {

        MsgV2Condition condition = createBaseCondition(userId, cuser, msgLife, fullUser, resultNotice, platforms);
        MsgBody body = new MsgBody();
        body.setMsgType(6);
        body.setLocationData(data);
        condition.setMsgElement(body);
        return condition;
    }

    /**
     * 创建地理位置消息
     *
     * @param data   地理位置信息
     * @param userId 用户id
     * @param cuser  是否是c端用户
     * @return 消息参数
     */
    public static MsgV2Condition createLocationCondition(HermesTriple<String, BigDecimal, BigDecimal> data,
                                                         Long userId,
                                                         boolean cuser) {
        return createLocationCondition(data, userId, cuser, 1);
    }

    /**
     * 创建地理位置消息
     *
     * @param data      地理位置信息
     * @param userId    用户id
     * @param cuser     是否是c端用户
     * @param platforms 平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createLocationCondition(HermesTriple<String, BigDecimal, BigDecimal> data,
                                                         Long userId,
                                                         boolean cuser,
                                                         Integer... platforms) {
        return createLocationCondition(data, userId, cuser, 604_800, false, true, platforms);
    }

    /**
     * 创建自定义消息
     *
     * @param data         自定义消息内容
     * @param userId       用户id
     * @param cuser        是否是c端用户
     * @param msgLife      消息最大保留时长（预留）
     * @param fullUser     是否全部用户
     * @param resultNotice 是否需要结果通知
     * @param platforms    平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createCustomCondition(Map<String, Object> data,
                                                       Long userId,
                                                       boolean cuser,
                                                       Integer msgLife,
                                                       boolean fullUser,
                                                       boolean resultNotice,
                                                       Integer... platforms) {

        MsgV2Condition condition = createBaseCondition(userId, cuser, msgLife, fullUser, resultNotice, platforms);
        MsgBody body = new MsgBody();
        body.setMsgType(9);
        body.setCustomData(data);
        condition.setMsgElement(body);
        return condition;
    }

    /**
     * 创建自定义消息
     *
     * @param data   自定义消息内容
     * @param userId 用户id
     * @param cuser  是否是c端用户
     * @return 消息参数
     */
    public static MsgV2Condition createCustomCondition(Map<String, Object> data,
                                                       Long userId,
                                                       boolean cuser) {
        return createCustomCondition(data, userId, cuser, 1);
    }

    /**
     * 创建自定义消息
     *
     * @param data      自定义消息内容
     * @param userId    用户id
     * @param cuser     是否是c端用户
     * @param platforms 平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createCustomCondition(Map<String, Object> data,
                                                       Long userId,
                                                       boolean cuser,
                                                       Integer... platforms) {
        return createCustomCondition(data, userId, cuser, 604_800, false, true, platforms);
    }


    /**
     * 创建基础消息对象
     *
     * @param userId       用户id
     * @param cuser        是否是c端用户
     * @param msgLife      消息最大保留时长（预留）
     * @param fullUser     是否全部用户
     * @param resultNotice 是否需要结果通知
     * @param platforms    平台 {@link cn.fw.hermes.sdk.v2.enums.Platform}
     * @return 消息参数
     */
    public static MsgV2Condition createBaseCondition(Long userId,
                                                     boolean cuser,
                                                     Integer msgLife,
                                                     boolean fullUser,
                                                     boolean resultNotice,
                                                     Integer... platforms) {
        MsgV2Condition condition = new MsgV2Condition();
        condition.setUserId(userId);
        condition.setMsgLife(msgLife);
        if (msgLife == null) {
            condition.setMsgLife(604_800);
        }
        condition.setUserType(1);
        if (cuser) {
            condition.setUserType(2);
        }
        condition.setFullUser(fullUser);
        condition.setResultNotice(resultNotice);
        List<Integer> platformList = Arrays.stream(platforms).collect(Collectors.toList());
        if (!platformList.isEmpty()) {
            condition.setPlatforms(platformList);
        }
        return condition;
    }

    public boolean checkPlatform() {
        if (platforms != null && !platforms.isEmpty()) {
            boolean result = true;
            for (int i = 0; i < platforms.size(); i++) {
                Integer platform = platforms.get(i);
                Platform value = Platform.ofValue(platform);
                result = Objects.nonNull(value);
                if (!result) {
                    break;
                }
            }
            return result;
        }
        return true;
    }

    public boolean checkUser() {
        if (Boolean.TRUE.equals(fullUser)) {
            return true;
        }
        AccountType accountType = AccountType.ofValue(userType);
        return Objects.nonNull(userId) && Objects.nonNull(accountType);
    }
}
