package cn.fw.hermes.task

import cn.fw.common.util.logError
import cn.fw.hermes.common.constant.Constant
import cn.fw.hermes.common.utils.ThreadPoolUtil
import cn.fw.hermes.domain.db.MessageData
import cn.fw.hermes.domain.dto.MessageConditionDTO
import cn.fw.hermes.service.biz.MsgPrepareBizService
import cn.fw.hermes.service.data.MessageDataService
import cn.hutool.json.JSONUtil
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.data.redis.core.StringRedisTemplate
import org.springframework.scheduling.annotation.Async
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Component
import org.springframework.util.CollectionUtils
import java.util.*
import java.util.concurrent.CompletableFuture

/**
 * 消息储存任务
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : 消息储存任务
 * @date : 2023-07-24 15:05
 */
@Component
@ConditionalOnProperty(prefix = "task", name = ["switch"], havingValue = "on")
class MessageSaveTask(
    private val stringRedisTemplate: StringRedisTemplate,
    private val msgPrepareBizService: MsgPrepareBizService,
    private val messageDataService: MessageDataService
) {
    /**
     * 重试保存失败的消息体
     */
    @Scheduled(initialDelay = 1000 * 5, fixedRate = 1000 * 10)
    fun retrySaveMsgBody() {
        val failList: MutableList<String> = mutableListOf()
        var jsonStr: String?
        while ((stringRedisTemplate.opsForSet().pop(Constant.MSG_BODY_SAVE_FAILED_CACHE_KEY).also { jsonStr = it }) != null) {
            val jsonArray = JSONUtil.parseArray(jsonStr)
            val list = JSONUtil.toList(jsonArray, MessageData::class.java)
            try {
                messageDataService.saveBatch(list)
            } catch (e: Exception) {
                jsonStr?.let { failList.add(it) }
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            val array = failList.toTypedArray<String>()
            stringRedisTemplate.opsForSet().add(Constant.MSG_BODY_SAVE_FAILED_CACHE_KEY, *array)
        }
    }

    /**
     * 持久化消息数据
     */
    @Async("taskExecutor")
    @Scheduled(initialDelay = 1000 * 5, fixedRate = 1000)
    fun saveMsg() {
        val failList: MutableList<String> = mutableListOf()
        val futureList: MutableList<CompletableFuture<Void>> = mutableListOf()
        var jsonStr: String?

        while ((stringRedisTemplate.opsForList().leftPop(Constant.CACHE_PERSISTENT_MSG_CACHE_KEY).also { jsonStr = it }) != null) {
            val dto = JSONUtil.toBean(jsonStr, MessageConditionDTO::class.java)
            if (Objects.isNull(dto)) {
                continue
            }
            val executor = ThreadPoolUtil.executor
            val future = CompletableFuture.runAsync({
                try {
                    msgPrepareBizService.saveMsg(dto)
                    stringRedisTemplate.opsForSet().add(Constant.READY_TO_SEND_MSG_CACHE_KEY, dto.id.toString())
                } catch (e: Exception) {
                    jsonStr?.let {
                        failList.add(it)
                    }

                    logError("持久化消息记录失败: [{}]", e.message)
                }
            }, executor)
            futureList.add(future)
        }

        CompletableFuture.allOf(*futureList.toTypedArray<CompletableFuture<*>>()).join()
        if (!CollectionUtils.isEmpty(failList)) {
            stringRedisTemplate.opsForList().rightPushAll(Constant.CACHE_PERSISTENT_MSG_CACHE_KEY, failList)
        }
    }
}
