package cn.fw.hermes.task

import cn.fw.common.util.logError
import cn.fw.hermes.common.constant.Constant
import cn.fw.hermes.domain.db.MessageHistory
import cn.fw.hermes.domain.enums.MsgStatusEnum
import cn.fw.hermes.service.biz.AppUpdateBizService
import cn.fw.hermes.service.biz.MessageSender
import cn.fw.hermes.service.data.MessageHistoryService
import com.baomidou.mybatisplus.core.toolkit.Wrappers
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.data.redis.core.StringRedisTemplate
import org.springframework.scheduling.annotation.Async
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Component
import org.springframework.util.CollectionUtils
import java.time.LocalDateTime

/**
 * 消息发送定时器
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : 消息发送定时器
 * @date : 2023-07-24 15:44
 */
@Component
@ConditionalOnProperty(prefix = "task", name = ["switch"], havingValue = "on")
class MessageSendTask(
    private val stringRedisTemplate: StringRedisTemplate,
    private val messageHistoryService: MessageHistoryService,
    private val messageSender: MessageSender,
    private val appUpdateBizService: AppUpdateBizService
) {
    @Scheduled(cron = "0/30 * * * * *")
    fun ready2Send() {
        val list = messageHistoryService.list(
            Wrappers.lambdaQuery<MessageHistory>()
                .eq(MessageHistory::getMsgStatus, MsgStatusEnum.WAITING)
                .ge(MessageHistory::getCreateTime, LocalDateTime.now().minusDays(7L))
        )
        if (CollectionUtils.isEmpty(list)) {
            return
        }

        val array = list.map {
            it.id.toString()
        }.distinct().toTypedArray()

        stringRedisTemplate.opsForSet().add(Constant.READY_TO_SEND_MSG_CACHE_KEY, *array)
    }

    /**
     * 推送消息
     */
    @Async("taskExecutor")
    @Scheduled(cron = "0/2 * * * * *")
    fun sendMsg() {
        val failList: MutableList<String> = mutableListOf()
        // 限流标志 [最多一次性推送100条]
        var count = 100
        var jsonStr: String?
        while ((stringRedisTemplate.opsForSet().pop(Constant.READY_TO_SEND_MSG_CACHE_KEY).also { jsonStr = it }) != null) {
            count--
            try {
                jsonStr?.toLong()?.let {
                    messageSender.sendMsg(it)
                }
            } catch (e: Exception) {
                jsonStr?.let { failList.add(it) }
                logError("推送消息失败", e)
            }
            if (count < 0) {
                break
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            stringRedisTemplate.opsForSet().add(Constant.READY_TO_SEND_MSG_CACHE_KEY, *failList.toTypedArray<String>())
        }
    }

    @Scheduled(cron = "0/20 * * * * *")
    fun reSendUpdateMsg() {
        appUpdateBizService.reSendUpdateMsg()
    }
}
