package cn.fw.hermes.service.biz;

import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.PassportAuthBean;
import cn.fw.hermes.common.constant.Constant;
import cn.fw.hermes.common.utils.StringUtils;
import cn.fw.hermes.common.utils.TemplateHelper;
import cn.fw.hermes.common.utils.UUIDUtils;
import cn.fw.hermes.domain.db.Hermes;
import cn.fw.hermes.domain.db.SysTopic;
import cn.fw.hermes.domain.enums.AccountStateEnum;
import cn.fw.hermes.domain.enums.AccountTypeEnum;
import cn.fw.hermes.domain.vo.AccountResultVO;
import cn.fw.hermes.rpc.member.MemberRpcService;
import cn.fw.hermes.rpc.member.dto.MemberUserDTO;
import cn.fw.hermes.service.data.HermesService;
import cn.fw.hermes.service.data.SysTopicService;
import cn.hutool.core.lang.Dict;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;


/**
 * @Author: Chenery
 * @Date: 2021/1/27 16:21
 */
@RequiredArgsConstructor
@Slf4j
@Service
public class AccountBizService {

    private final SysTopicService sysTopicService;
    private final HermesService hermesService;
    private final MemberRpcService memberRpcService;

    /**
     * B端用户查询topic
     *
     * @param user     用户信息
     * @param platform 平台
     * @return
     */
    public AccountResultVO queryStaffTopic(LoginAuthBean user, String platform) {
        Long userId = user.getUserId();
        String userName = user.getUserName();
        return queryStaffTopic(userId, userName, platform);
    }

    /**
     * B端用户查询topic
     * 查询用户topic和用户标识码，不存在则会注册
     *
     * @param userId   用户id
     * @param userName 用户名称
     * @param platform 平台
     * @return
     */
    public AccountResultVO queryStaffTopic(Long userId, String userName, String platform) {
        Hermes hermes = hermesService.queryByUserId(userId, AccountTypeEnum.B_USER);
        if (Objects.isNull(hermes)) {
            hermes = genHermes(userId, userName, AccountTypeEnum.B_USER);
            hermesService.save(hermes);
        }
        AccountResultVO vo = new AccountResultVO();
        vo.setIdentifier(hermes.getIdentifier());
        vo.setUserId(hermes.getUserId());
        String clientId = String.format("%s-%s", platform, hermes.getIdentifier());
        if (StringUtils.isValid(platform)) {
            renderTopics(vo, platform);
        }
        vo.setClientId(clientId);
        vo.setNickName(userName);
        return vo;
    }

    /**
     * C端用户查询topic
     *
     * @param user     用户信息
     * @param platform 平台
     * @return
     */
    public AccountResultVO queryUserTopic(PassportAuthBean user, String platform) {
        Long userId = user.getUserId();
        return queryUserTopic(userId, null, platform);
    }

    /**
     * C端用户查询topic
     * 查询用户topic和用户标识码，不存在则会注册
     *
     * @param userId   用户id
     * @param userName 用户名称
     * @param platform 平台
     * @return
     */
    public AccountResultVO queryUserTopic(Long userId, String userName, String platform) {
        String name = userName;
        if (StringUtils.isEmpty(userName)) {
            MemberUserDTO member = memberRpcService.user(userId);
            BV.notNull(member, () -> "用户不存在");
            name = member.getRealName();
        }
        Hermes hermes = hermesService.queryByUserId(userId, AccountTypeEnum.C_USER);
        if (Objects.isNull(hermes)) {
            hermes = genHermes(userId, name, AccountTypeEnum.C_USER);
            hermesService.save(hermes);
        }
        AccountResultVO vo = new AccountResultVO();
        vo.setIdentifier(hermes.getIdentifier());
        vo.setUserId(vo.getUserId());
        if (StringUtils.isValid(platform)) {
            renderTopics(vo, platform);
        }
        vo.setNickName(userName);
        vo.setClientId(String.format("%s-%s", platform, hermes.getIdentifier()));
        return vo;
    }

    /**
     * 将用户踢下线
     *
     * @param userId    用户ID
     * @param staffUser 是否是b端用户
     * @return 踢下线是否成功
     */
    public void kick(@NonNull Long userId, boolean staffUser) {
        // todo: implement
    }

    /**
     * 查询用户唯一标识码
     *
     * @param userId      用户ID
     * @param accountType 用户类型
     * @return
     */
    public String getUserIdentifier(@NonNull Long userId, AccountTypeEnum accountType) {
        Hermes hermes = hermesService.queryByUserId(userId, accountType);
        BV.notNull(hermes, () -> "用户信息不存在");
        return hermes.getIdentifier();
    }

    private Hermes genHermes(Long userId, String userName, AccountTypeEnum accountTypeEnum) {
        Hermes hermes = new Hermes();
        hermes.setIdentifier(UUIDUtils.INSTANCE.newStaffIdentifier(userId));
        hermes.setUserId(userId);
        hermes.setNickName(userName);
        hermes.setXgToken(Constant.INVALID_TOKEN);
        hermes.setAccountType(accountTypeEnum);
        hermes.setState(AccountStateEnum.NORMAL);
        hermes.setCreateDay(LocalDate.now());
        return hermes;
    }

    private void renderTopics(AccountResultVO vo, String platform) {
        final TemplateHelper helper = TemplateHelper.INSTANCE;
        final Dict dict = Dict.create();
        dict.set("identifier", vo.getIdentifier());
        dict.set("platform", platform);
        List<SysTopic> sysTopics = sysTopicService.queryTopics();
        if (!CollectionUtils.isEmpty(sysTopics)) {
            List<String> topicList = sysTopics.stream().map(r -> helper.render(r.getTopic(), dict)).collect(Collectors.toList());
            vo.setTopics(topicList);
        }
    }
}
