package cn.fw.hermes.service.biz;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.common.cache.locker.FLock;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.hermes.common.constant.Constant;
import cn.fw.hermes.domain.db.AppUpdateMsg;
import cn.fw.hermes.domain.db.SysPlatform;
import cn.fw.hermes.domain.enums.MsgStatusEnum;
import cn.fw.hermes.domain.enums.PlatformTypeEnum;
import cn.fw.hermes.domain.mqtt.UpdateMessage;
import cn.fw.hermes.sdk.v2.para.AppUpdateMsgCondition;
import cn.fw.hermes.service.data.AppUpdateMsgService;
import cn.fw.hermes.service.data.SysPlatformService;
import cn.fw.hermes.service.emqx.MqttService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * 应用更新
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : 应用更新
 * @date : 2023-08-14 10:22
 */
@Slf4j
@Service
@RefreshScope
@RequiredArgsConstructor
public class AppUpdateBizService {
    private final AppUpdateMsgService appUpdateMsgService;
    private final SysPlatformService sysPlatformService;
    private final MqttService mqttService;
    private final DistributedLocker distributedLocker;

    @Value("${spring.cache.locker.key-prefix}:UpdateMsg")
    @Getter
    private String keyPrefix;

    @DisLock(prefix = "#this.getKeyPrefix()", key = "#condition.platform", message = "请勿重复发送")
    @Transactional(rollbackFor = Exception.class)
    public void sendUpdateMsg(AppUpdateMsgCondition condition) {
        AppUpdateMsg appUpdateMsg = generateUpdateMsg(condition.getPlatform(), condition);
        String platformCode = appUpdateMsg.getPlatformCode();
        Integer versionCode = appUpdateMsg.getVersionCode();
        List<AppUpdateMsg> list = appUpdateMsgService.list(Wrappers.<AppUpdateMsg>lambdaQuery()
                .eq(AppUpdateMsg::getPlatformCode, platformCode)
                .eq(AppUpdateMsg::getLatest, Boolean.TRUE)
                .lt(AppUpdateMsg::getVersionCode, versionCode)
        );
        if (!CollectionUtils.isEmpty(list)) {
            List<Long> idList = list.stream().map(AppUpdateMsg::getId).collect(Collectors.toList());
            appUpdateMsgService.updateLatestFlag(idList);
        }
        sendUpdateMsg(appUpdateMsg);
        appUpdateMsgService.save(appUpdateMsg);
    }

    /**
     * 重发更新消息
     */
    public void reSendUpdateMsg() {
        final String lockKey = getKeyPrefix() + ":resend";
        Pair<Boolean, FLock> lockPair = distributedLocker.tryLock(lockKey, TimeUnit.MINUTES, 0, -1);
        try {
            if (Boolean.TRUE.equals(lockPair.getLeft())) {
                List<AppUpdateMsg> list = appUpdateMsgService.list(Wrappers.<AppUpdateMsg>lambdaQuery()
                        .in(AppUpdateMsg::getStatus, MsgStatusEnum.WAITING, MsgStatusEnum.FAIL)
                        .eq(AppUpdateMsg::getLatest, Boolean.TRUE)
                );
                if (CollectionUtils.isEmpty(list)) {
                    return;
                }
                for (AppUpdateMsg appUpdateMsg : list) {
                    boolean isSuccess = this.sendUpdateMsg(appUpdateMsg);
                    if (isSuccess) {
                        appUpdateMsgService.updateById(appUpdateMsg);
                    }
                }
            }
        } catch (Exception e) {
            log.error("重发更新消息失败", e);
        } finally {
            if (lockPair.getLeft()) {
                lockPair.getRight().unlock();
            }
        }
    }

    /**
     * debug test app update message
     *
     * @param versionCode
     * @param force
     */
    public void sendDebugUpdateMsg(Integer versionCode, Boolean force) {
        AppUpdateMsg appUpdateMsg = generateDebugUpdateMsg(versionCode, force);
        UpdateMessage msg = generateUpdateMessage(appUpdateMsg);
        String topic = BeanTransfer.renderTopic(Constant.PLATFORM_TOPIC, "", appUpdateMsg.getPlatformCode());
        final Pair<Boolean, String> pair = mqttService.senUpdateMsg(topic, msg);
        BV.notNull(pair, () -> "发送失败，mqtt未连接");
        BV.isTrue(pair.getLeft(), pair::getRight);
    }

    private boolean sendUpdateMsg(AppUpdateMsg updateMsg) {
        UpdateMessage msg = generateUpdateMessage(updateMsg);
        String topic = BeanTransfer.renderTopic(Constant.PLATFORM_TOPIC, "", updateMsg.getPlatformCode());
        Pair<Boolean, String> pair = mqttService.senUpdateMsg(topic, msg);
        if (Objects.nonNull(pair)) {
            boolean isSuccess = Boolean.TRUE.equals(pair.getKey());
            updateMsg.setStatus(isSuccess ? MsgStatusEnum.SUCCESS : MsgStatusEnum.FAIL);
            return true;
        }
        return false;
    }

    private AppUpdateMsg generateDebugUpdateMsg(Integer versionCode, Boolean force) {
        SysPlatform sysPlatform = sysPlatformService.queryPlatformByType(PlatformTypeEnum.APP);
        AppUpdateMsg appUpdateMsg = new AppUpdateMsg();
        appUpdateMsg.setPlatformCode(sysPlatform.getPlatformCode());
        appUpdateMsg.setStatus(MsgStatusEnum.WAITING);
        appUpdateMsg.setRemark(Constant.UPDATE_MSG);

        appUpdateMsg.setUpdateRemark("1、系统优化 \n2、性能优化 \n3、修复一些bug");
        appUpdateMsg.setForceUpload(Boolean.TRUE.equals(force));
        appUpdateMsg.setTargetUrl("https://feewee-1255997270.cos.ap-chengdu.myqcloud.com/app/2/748/3.1.0_230811admin1691748739334_%E9%9C%8F%E8%BD%A6%E8%BD%A6%E9%9B%86%E5%9B%A2%E7%89%8808111756_release.apk");
        appUpdateMsg.setVersionCode(versionCode);
        appUpdateMsg.setVersionName("debug_3.1.0_230811");
        appUpdateMsg.setMd5("B555B659E58E973002D4979F0E0C7B67");
        appUpdateMsg.setFileSize(98227390);
        return appUpdateMsg;
    }

    private AppUpdateMsg generateUpdateMsg(Integer platform, AppUpdateMsgCondition condition) {
        PlatformTypeEnum platformTypeEnum = BeanTransfer.transFromSdkPlatformType(platform);
        SysPlatform sysPlatform = sysPlatformService.queryPlatformByType(platformTypeEnum);
        AppUpdateMsg appUpdateMsg = new AppUpdateMsg();
        appUpdateMsg.setPlatformCode(sysPlatform.getPlatformCode());
        appUpdateMsg.setStatus(MsgStatusEnum.WAITING);
        appUpdateMsg.setRemark(Constant.UPDATE_MSG);

        appUpdateMsg.setUpdateRemark(condition.getUpdateRemark());
        appUpdateMsg.setForceUpload(Boolean.TRUE.equals(condition.getForceUpload()));
        appUpdateMsg.setTargetUrl(condition.getTargetUrl());
        appUpdateMsg.setVersionCode(condition.getVersionCode());
        appUpdateMsg.setVersionName(condition.getVersionName());
        appUpdateMsg.setMd5(condition.getMd5());
        appUpdateMsg.setFileSize(condition.getFileSize());
        appUpdateMsg.setLatest(Boolean.TRUE);
        return appUpdateMsg;
    }

    private UpdateMessage generateUpdateMessage(AppUpdateMsg appUpdateMsg) {
        UpdateMessage msg = new UpdateMessage();
        msg.setRemark(appUpdateMsg.getRemark());
        msg.setUpdateRemark(appUpdateMsg.getUpdateRemark());
        msg.setForceUpload(appUpdateMsg.getForceUpload());
        msg.setTargetUrl(appUpdateMsg.getTargetUrl());
        msg.setVersionCode(appUpdateMsg.getVersionCode());
        msg.setVersionName(appUpdateMsg.getVersionName());
        msg.setMd5(appUpdateMsg.getMd5());
        msg.setFileSize(appUpdateMsg.getFileSize());
        return msg;
    }
}
