package cn.fw.hermes.service.biz;

import cn.fw.hermes.common.constant.Constant;
import cn.fw.hermes.common.utils.StringUtils;
import cn.fw.hermes.common.utils.TemplateHelper;
import cn.fw.hermes.domain.db.MessageData;
import cn.fw.hermes.domain.db.MessageHistory;
import cn.fw.hermes.domain.dto.MessageBodyDTO;
import cn.fw.hermes.domain.dto.MessageConditionDTO;
import cn.fw.hermes.domain.enums.AccountTypeEnum;
import cn.fw.hermes.domain.enums.MsgStatusEnum;
import cn.fw.hermes.domain.enums.MsgTypeEnum;
import cn.fw.hermes.domain.enums.PlatformTypeEnum;
import cn.fw.hermes.sdk.v2.enums.AccountType;
import cn.fw.hermes.sdk.v2.enums.MsgState;
import cn.fw.hermes.sdk.v2.enums.MsgType;
import cn.fw.hermes.sdk.v2.enums.Platform;
import cn.fw.hermes.sdk.v2.para.HermesPair;
import cn.fw.hermes.sdk.v2.para.HermesTriple;
import cn.fw.hermes.sdk.v2.para.MsgBody;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.lang.Dict;
import cn.hutool.json.JSONUtil;
import org.springframework.lang.NonNull;

import java.math.BigDecimal;
import java.util.Map;
import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * sdk <=> dto类型互转类
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : sdk <=> dto类型互转类
 * @date : 2023-07-24 09:52
 */
public final class BeanTransfer {

    /**
     * sdk类型消息数据转dto类型的消息数据
     *
     * @param element sdk类型消息
     * @return dto类型的消息数据
     */
    public static MessageBodyDTO transform2MsgBody(MsgBody element) {
        MessageBodyDTO dto = new MessageBodyDTO();
        MsgTypeEnum msgType = BeanTransfer.transFromSdkMsgType(element.getMsgType());
        dto.setType(msgType.getValue());
        switch (msgType) {
            case TEXT:
                dto.setText(element.getText());
                break;
            case IMAGE:
                HermesTriple<String, String, String> imageData = element.getImageData();
                String original = imageData.getRight();
                String middle = imageData.getMiddle();
                String thumbnail = imageData.getLeft();
                if (StringUtils.isEmpty(middle)) {
                    middle = original;
                }
                if (StringUtils.isEmpty(thumbnail)) {
                    thumbnail = original;
                }
                dto.setImageUrl(ListUtil.toList(thumbnail, middle, original));
                break;
            case VOICE:
                HermesPair<String, Integer> voiceData = element.getVoiceData();
                String voiceUrl = voiceData.getLeft();
                Integer voiceDuration = voiceData.getRight();
                dto.setVoiceUrl(voiceUrl);
                dto.setDuration(voiceDuration);
                break;
            case VIDEO:
                HermesTriple<String, String, Integer> videoData = element.getVideoData();
                String cover = videoData.getLeft();
                String videoUrl = videoData.getMiddle();
                Integer videoDuration = videoData.getRight();
                dto.setVideoUrl(ListUtil.toList(cover, videoUrl));
                dto.setDuration(videoDuration);
                break;
            case EMOJI:
                dto.setEmojiIndex(element.getEmojiIdx());
                break;
            case LOCATION:
                HermesTriple<String, BigDecimal, BigDecimal> locationData = element.getLocationData();
                dto.setText(locationData.getLeft());
                dto.setLat(locationData.getMiddle());
                dto.setLng(locationData.getRight());
                break;
            case CUSTOM:
                Map<String, Object> customData = element.getCustomData();
                String jsonStr = JSONUtil.toJsonStr(customData);
                dto.setData(jsonStr);
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + msgType);
        }

        return dto;
    }

    /**
     * dto -> db
     *
     * @param dto
     * @return
     */
    public static MessageHistory transform2MsgHistory(MessageConditionDTO dto) {
        MessageHistory history = new MessageHistory();
        history.setId(dto.getId());
        history.setXgTodo(false);
        MessageBodyDTO firstMsgBody = dto.getMsgBodyList().get(0);
        MsgTypeEnum msgTypeEnum = MsgTypeEnum.ofValue(firstMsgBody.getType());
        String format = "[%s]";
        switch (Objects.requireNonNull(msgTypeEnum)) {
            case TEXT:
                history.setMsgSummary(firstMsgBody.getText());
                break;
            case IMAGE:
                history.setMsgSummary(String.format(format, "图片"));
                break;
            case VOICE:
                history.setMsgSummary(String.format(format, "语音"));
                break;
            case VIDEO:
                history.setMsgSummary(String.format(format, "视频"));
                break;
            case EMOJI:
                history.setMsgSummary(String.format(format, "表情"));
                break;
            case LOCATION:
                history.setMsgSummary(String.format(format, "位置"));
                break;
            case CUSTOM:
                history.setMsgSummary(String.format(format, "其他"));
                String data = firstMsgBody.getData();
                boolean isBackLog = data.indexOf(Constant.BACKLOG_MSG) > 0;
                boolean hasApp = dto.getPlatforms().stream().anyMatch(t -> t.equals(PlatformTypeEnum.APP.getValue()));
                if (isBackLog && hasApp) {
                    history.setXgTodo(true);
                    history.setMsgSummary(String.format(format, "待办"));
                }
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + msgTypeEnum);
        }
        history.setMsgStatus(MsgStatusEnum.WAITING);
        if (Boolean.TRUE.equals(dto.getFullUser())) {
            history.setToAccount("all");
            history.setXgTodo(false);
        } else {
            history.setToAccount(String.valueOf(dto.getUserId()));
        }
        history.setFromAccount("admin");
        history.setNeedResult(dto.getNeedResult());
        history.setMsgLifeTime(dto.getMsgLifeTime());
        return history;
    }

    /**
     * 消息体转换
     *
     * @param id
     * @param dto
     * @return
     */
    public static MessageData transform2MsgData(Long id, MessageBodyDTO dto) {
        MessageData msgData = new MessageData();
        msgData.setMsgId(id);
        MsgTypeEnum msgTypeEnum = MsgTypeEnum.ofValue(dto.getType());
        msgData.setType(msgTypeEnum);
        msgData.setData(dto.getData());
        msgData.setText(dto.getText());
        msgData.setImageUrl(dto.getImageUrl());
        msgData.setEmojiIndex(dto.getEmojiIndex());
        msgData.setLat(dto.getLat());
        msgData.setLng(dto.getLng());
        msgData.setVoiceUrl(dto.getVoiceUrl());
        msgData.setVideoUrl(dto.getVideoUrl());
        msgData.setDuration(dto.getDuration());
        return msgData;
    }

    /**
     * sdk消息类型转换成db类型
     *
     * @param type sdk类型
     * @return db枚举类型
     */
    public static MsgTypeEnum transFromSdkMsgType(@NonNull cn.fw.hermes.sdk.v2.enums.MsgType type) {
        switch (type) {
            case TEXT:
                return MsgTypeEnum.TEXT;
            case IMAGE:
                return MsgTypeEnum.IMAGE;
            case VOICE:
                return MsgTypeEnum.VOICE;
            case VIDEO:
                return MsgTypeEnum.VIDEO;
            case EMOJI:
                return MsgTypeEnum.EMOJI;
            case LOCATION:
                return MsgTypeEnum.LOCATION;
            case CUSTOM:
                return MsgTypeEnum.CUSTOM;
            default:
                throw new IllegalStateException("Unexpected value: " + type);
        }
    }

    /**
     * sdk消息类型转换成db类型
     *
     * @param code sdk msgType enum code
     * @return db枚举类型
     */
    public static MsgTypeEnum transFromSdkMsgType(final int code) {
        MsgType type = MsgType.ofCode(code);
        BV.notNull(type, () -> "Unexpected value: " + code);
        return transFromSdkMsgType(Objects.requireNonNull(type));
    }

    /**
     * 账号类型转换
     *
     * @param type
     * @return
     */
    public static AccountTypeEnum transFromSdkAccountType(AccountType type) {
        switch (type) {
            case B_USER:
                return AccountTypeEnum.B_USER;
            case C_USER:
                return AccountTypeEnum.C_USER;
            default:
                throw new IllegalStateException("Unexpected value: " + type);
        }
    }

    public static AccountTypeEnum transFromSdkAccountType(final int value) {
        AccountType accountType = AccountType.ofValue(value);
        BV.notNull(accountType, () -> "Unexpected value: " + value);
        return transFromSdkAccountType(Objects.requireNonNull(accountType));
    }

    /**
     * 平台类型转换
     *
     * @param platform
     * @return
     */
    public static PlatformTypeEnum transFromSdkPlatformType(Platform platform) {
        switch (platform) {
            case APP:
                return PlatformTypeEnum.APP;
            case WEB:
                return PlatformTypeEnum.WEB;
            case REPORT:
                return PlatformTypeEnum.REPORT;
            case H5:
                return PlatformTypeEnum.H5;
            case MINI:
                return PlatformTypeEnum.MINI;
            case EVO:
                return PlatformTypeEnum.EVO;
            default:
                throw new IllegalStateException("Unexpected value: " + platform);
        }
    }

    public static PlatformTypeEnum transFromSdkPlatformType(final int value) {
        Platform platform = Platform.ofValue(value);
        return transFromSdkPlatformType(Objects.requireNonNull(platform));
    }

    public static MsgState trans2SdkMsgState(MsgStatusEnum status) {
        switch (status) {
            case WAITING:
                return MsgState.WAITING;
            case UPLOAD:
                return MsgState.SENDING;
            case SUCCESS:
                return MsgState.SUCCESS;
            case FAIL:
                return MsgState.FAIL;
            default:
                throw new IllegalStateException("Unexpected value: " + status);
        }
    }

    public static String renderTopic(String format, String identifier, String platformCode) {
        final TemplateHelper helper = TemplateHelper.INSTANCE;
        final Dict dict = Dict.create();
        dict.set("identifier", identifier);
        dict.set("platform", platformCode);
        return helper.render(format, dict);
    }
}
