package cn.fw.hermes.service.biz;

import cn.fw.common.cache.CacheExtender;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.hermes.common.constant.Constant;
import cn.fw.hermes.domain.db.MessageBusinessType;
import cn.fw.hermes.domain.db.SysPlatform;
import cn.fw.hermes.domain.db.SysTopic;
import cn.fw.hermes.domain.dto.PlatformDTO;
import cn.fw.hermes.domain.dto.TopicDTO;
import cn.fw.hermes.domain.vo.MsgBizTypeVO;
import cn.fw.hermes.domain.vo.PlatformVO;
import cn.fw.hermes.domain.vo.TopicVO;
import cn.fw.hermes.service.data.MessageBusinessTypeService;
import cn.fw.hermes.service.data.SysPlatformService;
import cn.fw.hermes.service.data.SysTopicService;
import cn.fw.hermes.service.emqx.MqttService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * 通用服务
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : 通用服务
 * @date : 2023-06-21 17:07
 */
@RequiredArgsConstructor
@Slf4j
@Service
@RefreshScope
public class CommonBizService {
    private final SysPlatformService sysPlatformService;
    private final SysTopicService sysTopicService;
    private final MessageBusinessTypeService messageBusinessTypeService;
    private final CacheExtender cache;
    private final MqttService mqttService;

    @Value("${spring.cache.locker.key-prefix}:common")
    @Getter
    private String keyPrefix;

    /**
     * 初始化系统数据
     */
    public void initSysData() {
        cache.clearCache(Constant.PLATFORM_CACHE_KEY);
        cache.clearCache(Constant.TOPIC_CACHE_KEY);
        sysPlatformService.queryPlatforms();
        sysTopicService.queryTopics();
    }

    /**
     * 查询所有主题
     *
     * @return
     */
    public List<TopicVO> queryTopics() {
        List<SysTopic> sysTopics = sysTopicService.queryTopics();
        return sysTopics.stream().map(TopicVO::with).collect(Collectors.toList());
    }

    /**
     * 查询所有平台
     *
     * @return
     */
    public List<PlatformVO> queryPlatforms() {
        List<SysPlatform> sysPlatforms = sysPlatformService.queryPlatforms();
        return sysPlatforms.stream().map(PlatformVO::with).collect(Collectors.toList());
    }

    /**
     * 查询所有自定义消息的业务类型
     *
     * @return
     */
    public List<MsgBizTypeVO> queryMsgBizTypes() {
        List<MessageBusinessType> list = messageBusinessTypeService.list();
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list.stream().map(r -> new MsgBizTypeVO(r.getId(), r.getType(), r.getTypeDesc())).collect(Collectors.toList());
    }

    /**
     * 修改和新增平台
     *
     * @param dto
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'platform'", message = "请勿重复操作")
    public void savePlatform(PlatformDTO dto) {
        sysPlatformService.modifyAndAdd(dto);
    }

    /**
     * 修改和新增主题
     *
     * @param dto
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'topic'", message = "请勿重复操作")
    public void saveTopic(TopicDTO dto) {
        sysTopicService.modifyAndAdd(dto);
    }

    /**
     * 新增一个自定义消息的业务类型
     *
     * @param type
     * @param desc
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'msg_biz_type'", message = "请勿重复操作")
    public void saveMsgBizType(String type, String desc) {
        boolean exist = messageBusinessTypeService.count(Wrappers.<MessageBusinessType>lambdaQuery()
            .eq(MessageBusinessType::getType, type)
        ) > 0;
        BV.isFalse(exist, () -> "已存在该类型");
        MessageBusinessType bizType = new MessageBusinessType();
        bizType.setType(type);
        bizType.setTypeDesc(desc);
        messageBusinessTypeService.save(bizType);
    }

    /**
     * 重连mqtt服务器
     *
     * @return
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'msg_client_reconnect'", message = "请勿重复操作")
    public boolean reconnect() {
        return mqttService.manualReConnect();
    }

    /**
     * 模拟发送测试预警钉钉机器人消息
     *
     * @return
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'debug_send_warning_msg'", message = "请勿重复操作")
    public void debugSendWarningMsg() {
        mqttService.sendWarningMsg();
    }
}
