package cn.fw.hermes.service.biz

import cn.fw.hermes.common.constant.Constant
import cn.fw.hermes.common.utils.DateUtil
import cn.fw.hermes.domain.db.MessageData
import cn.fw.hermes.domain.db.MessageHistory
import cn.fw.hermes.domain.enums.AccountTypeEnum
import cn.fw.hermes.domain.enums.MsgStatusEnum
import cn.fw.hermes.domain.enums.MsgTypeEnum
import cn.fw.hermes.domain.enums.PlatformTypeEnum
import cn.fw.hermes.domain.mqtt.Message
import cn.fw.hermes.domain.mqtt.msgbody.*
import cn.fw.hermes.sdk.v2.enums.MsgState
import cn.fw.hermes.sdk.v2.result.MessageV2Result
import cn.fw.hermes.service.data.DisturbSettingService
import cn.fw.hermes.service.data.MessageDataService
import cn.fw.hermes.service.data.MessageHistoryService
import cn.fw.hermes.service.data.SysPlatformService
import cn.fw.hermes.service.emqx.MqttService
import cn.fw.hermes.service.mq.MessageProducer
import org.apache.commons.lang3.tuple.Pair
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.util.CollectionUtils
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.LocalTime
import java.time.ZoneOffset
import java.util.*
import java.util.stream.Collectors

/**
 * 消息发送服务
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : 消息发送服务
 * @date : 2023-07-21 15:11
 */
@Service
class MessageSender(
    private val messageHistoryService: MessageHistoryService,
    private val messageDataService: MessageDataService,
    private val mqttService: MqttService,
    private val messageProducer: MessageProducer,
    private val xgPushService: XgPushService,
    private val sysPlatformService: SysPlatformService,
    private val disturbSettingService: DisturbSettingService
) {

    @Transactional(rollbackFor = [Exception::class])
    fun sendMsg(messageId: Long?) {
        val history = messageHistoryService.getById(messageId)
        history?.takeIf { MsgStatusEnum.WAITING == it.msgStatus }?.run {
            val dataList = messageDataService.queryDataByMsgId(history.id)
            if (CollectionUtils.isEmpty(dataList)) {
                return
            }
            val pair = sendMsg(history, dataList)
            if (Objects.isNull(pair) || MsgStatusEnum.UPLOAD == pair?.left) {
                return
            }
            if (history.needResult == true) {
                val result = MessageV2Result()
                result.setMessageId(history.id)
                result.setMsgState(MsgState.FAIL.value)
                result.setMsgStateDesc(history.msgStatusDesc)
                result.setResultTime(DateUtil.toDate(LocalDateTime.now()))
                messageProducer.send(result)
            }
        }
    }

    @Transactional(rollbackFor = [Exception::class])
    fun sendMsg(history: MessageHistory, dataList: List<MessageData>): Pair<MsgStatusEnum, String>? {
        val xgTodo = history.xgTodo
        val userId = history.toAccount.toLong()
        val platformCode = sysPlatformService.queryPlatforms().find { it.platformType == PlatformTypeEnum.APP }?.run { platformCode } ?: Constant.INVALID_TOKEN
        val userDisturb = disturbSettingService.queryByUserPlatform(userId, platformCode)
        val useMqtt = userDisturb?.takeIf { it.enabled == true }?.run { this@MessageSender.isTimeInRange(startTime, endTime) } ?: false
        history.sendTime = LocalDateTime.now()
        history.sendDay = LocalDate.now()
        history.createTime = Date()
        if (useMqtt || !xgTodo) {
            return mqtSend(history, dataList)
        }
        return xgPushService.sendMsg(userId, AccountTypeEnum.B_USER).run {
            if (first == -1) {
                mqtSend(history, dataList)
            } else {
                val sendSuccess = first == 0
                history.sendSuccessTime = LocalDateTime.now()
                history.xgToken = second
                if (sendSuccess) {
                    history.msgStatus = MsgStatusEnum.SUCCESS
                    history.msgStatusDesc = Constant.SUCCESS_MSG
                    history.xgId = third
                } else {
                    history.msgStatus = MsgStatusEnum.FAIL
                    history.msgStatusDesc = "$first: $third"
                }
                messageHistoryService.saveOrUpdate(history)
                Pair.of(history.msgStatus, history.msgStatusDesc)
            }
        }
    }

    private fun mqtSend(
        history: MessageHistory,
        dataList: List<MessageData>
    ): Pair<MsgStatusEnum, String>? {
        val msg = history.toMsg()
        val list = dataList.stream().map { r: MessageData -> this.genElement(r, msg) }.collect(Collectors.toList())
        msg.msgBody = list
        val msgTopics = history.msgTopic
        val pair = mqttService.senMsg(msgTopics, msg)
        return pair?.let {
            val isSuccess = true == it.key
            val desc = it.value
            history.xgId = Constant.INVALID_TOKEN
            history.xgToken = Constant.INVALID_TOKEN
            history.msgStatus = MsgStatusEnum.UPLOAD
            history.msgStatusDesc = desc
            if (!isSuccess) {
                history.msgStatus = MsgStatusEnum.FAIL
                history.sendSuccessTime = LocalDateTime.now()
            }
            messageHistoryService.saveOrUpdate(history)
            Pair.of(history.msgStatus, desc)
        }
    }

    private fun genElement(msgData: MessageData, msg: Message): MsgElement? {
        val msgTypeEnum = msgData.type
        when (msgTypeEnum) {
            MsgTypeEnum.TEXT -> return TextMsg.generate(msgData.msgId, msgData.text)
            MsgTypeEnum.IMAGE -> return ImageMsg.generate(msgData.msgId, msgData.imageUrl)
            MsgTypeEnum.VOICE -> return VoiceMsg.generate(msgData.msgId, msgData.voiceUrl, msgData.duration)
            MsgTypeEnum.VIDEO -> return VideoMsg.generate(msgData.msgId, msgData.duration, msgData.videoUrl)
            MsgTypeEnum.EMOJI -> return EmojiMsg.generate(msgData.msgId, msgData.emojiIndex)
            MsgTypeEnum.LOCATION -> return LocationMsg.generate(msgData.msgId, msgData.text, msgData.lat, msgData.lng)
            MsgTypeEnum.CUSTOM -> {
                val data = msgData.data
                val isBackLog = data.indexOf(Constant.BACKLOG_MSG) > 0
                if (isBackLog) {
                    msg.remark = Constant.BACKLOG_MSG
                }
                return CustomMsg.generate(msgData.msgId, data)
            }

            else -> {}
        }
        return null
    }

    private fun isTimeInRange(startTime: LocalTime, endTime: LocalTime): Boolean {
        val currentTime = LocalTime.now()
        if (endTime.isBefore(startTime)) {
            return !currentTime.isBefore(startTime) || currentTime.isBefore(endTime)
        }
        return currentTime.isAfter(startTime) && currentTime.isBefore(endTime)
    }

    private fun MessageHistory.toMsg(): Message {
        val msg = Message()
        msg.msgId = this.id
        msg.msgSummary = this.msgSummary
        msg.sender = this.fromAccount
        msg.receiver = this.toAccount
        msg.sendTimestamp = this.sendTime.toInstant(ZoneOffset.of("+8")).toEpochMilli()
        msg.msgTimestamp = this.createTime.time
        return msg
    }
}
