package cn.fw.hermes.service.biz

import cn.fw.common.businessvalidator.Validator.BV
import cn.fw.common.util.logError
import cn.fw.hermes.common.constant.Constant
import cn.fw.hermes.common.utils.DateUtil
import cn.fw.hermes.domain.db.MessageData
import cn.fw.hermes.domain.db.MessageHistory
import cn.fw.hermes.domain.dto.MessageBodyDTO
import cn.fw.hermes.domain.dto.MessageConditionDTO
import cn.fw.hermes.domain.dto.MsgDebugDTO
import cn.fw.hermes.domain.enums.PlatformTypeEnum
import cn.fw.hermes.sdk.v2.para.MsgV2Condition
import cn.fw.hermes.sdk.v2.result.MessageV2Result
import cn.fw.hermes.service.data.HermesService
import cn.fw.hermes.service.data.MessageDataService
import cn.fw.hermes.service.data.MessageHistoryService
import cn.fw.hermes.service.data.SysPlatformService
import cn.hutool.json.JSONUtil
import org.apache.commons.lang3.tuple.Pair
import org.springframework.data.redis.core.StringRedisTemplate
import org.springframework.lang.NonNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.util.CollectionUtils
import java.time.LocalDate
import java.time.LocalDateTime
import java.util.*
import java.util.stream.Collectors

/**
 * 消息业务类
 * 发送前的准备
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : 消息业务类
 * @date : 2023-06-21 15:52
 */
@Service
class MsgPrepareBizService(
    private val stringRedisTemplate: StringRedisTemplate,
    private val hermesService: HermesService,
    private val messageHistoryService: MessageHistoryService,
    private val messageDataService: MessageDataService,
    private val sysPlatformService: SysPlatformService,
    private val messageSender: MessageSender
) {

    /**
     * 发送异步消息
     *
     * @param condition 消息对象
     * @return 消息id
     */
    fun asyncSendMsg(@NonNull condition: MsgV2Condition): Long {
        val dto = translateCondition(condition)
        val jsonStr = JSONUtil.toJsonStr(dto)
        stringRedisTemplate.opsForList().rightPush(Constant.CACHE_PERSISTENT_MSG_CACHE_KEY, jsonStr)
        return dto.id
    }

    /**
     * 发送同步消息
     *
     * @param condition 消息对象
     * @return 消息id
     */
    fun sendMsg(@NonNull condition: MsgV2Condition): MessageV2Result {
        val dto = translateCondition(condition)
        val beanPair = generateDbBean(dto)
        val messageDataList = beanPair.right
        val pair = messageSender.sendMsg(beanPair.left, messageDataList)
        BV.notNull(pair) { "发送失败，服务未连接" }
        try {
            messageDataService.saveBatch(messageDataList)
        } catch (e: Exception) {
            logError("保存消息内容失败", e)
            val jsonArray = JSONUtil.parseArray(messageDataList)
            stringRedisTemplate.opsForList().rightPush(Constant.MSG_BODY_SAVE_FAILED_CACHE_KEY, jsonArray.toString())
        }
        val result = MessageV2Result()
        result.setMessageId(dto.id)
        val msgState = BeanTransfer.trans2SdkMsgState(pair!!.left)
        result.setMsgState(msgState.value)
        result.setMsgStateDesc(pair.value)
        result.setResultTime(DateUtil.toDate(LocalDateTime.now()))
        return result
    }

    /**
     * 消息存入数据库
     *
     * @param dto
     */
    @Transactional(rollbackFor = [Exception::class])
    fun saveMsg(dto: MessageConditionDTO) {
        val beanPair = generateDbBean(dto)
        val history = beanPair.left
        val messageId = history.id
        val messageHistory = messageHistoryService.getById(messageId)
        if (Objects.nonNull(messageHistory)) {
            return
        }
        messageHistoryService.save(history)
        messageDataService.saveBatch(beanPair.right)
    }

    /**
     * debug发送消息方法
     * @param dto
     */
    fun debugSendMsg(dto: MsgDebugDTO) {
        val data: MutableMap<String, Any> = HashMap()
        data["type"] = "BacklogItemNotice"
        val map = HashMap<String, Any>()
        map["itemName"] = "待办测试"
        map["itemCode"] = "NZy7QwFYbZ"
        map["type"] = 1
        map["pageParam"] = HashMap<Any, Any>()
        map["listPage"] = 111
        data["type"] = "BacklogItemNotice"
        data["data"] = map
        val condition = MsgV2Condition.createCustomCondition(data, dto.userId, false, dto.platform)
        this.sendMsg(condition)
    }


    private fun generateDbBean(dto: MessageConditionDTO): Pair<MessageHistory, List<MessageData>> {
        val id = dto.id
        val userId = dto.userId
        val userType = dto.userType
        val fullUser = (true == dto.fullUser)
        // 处理平台值，取第一项默认为app平台
        val platform = dto.platforms?.getOrNull(0) ?: PlatformTypeEnum.APP.value
        dto.platforms = arrayListOf(platform)
        val history = BeanTransfer.transform2MsgHistory(dto)
        val topicList = generateTopic(userId, userType, dto.platforms, fullUser)
        history.setMsgTopic(topicList)
        val messageDataList = dto.msgBodyList.stream().map { r: MessageBodyDTO? ->
            BeanTransfer.transform2MsgData(
                id,
                r
            )
        }.collect(Collectors.toList())
        return Pair.of(history, messageDataList)
    }

    private fun translateCondition(condition: MsgV2Condition): MessageConditionDTO {
        val dto = MessageConditionDTO()
        val today = LocalDate.now()
        val uniqueCacheKey = Constant.UNIQUE_ID_CACHE_KEY.apply(today.toString())
        val increment = stringRedisTemplate.opsForValue().increment(uniqueCacheKey)
        BV.notNull(increment) { "自增id生成失败" }
        val idStr = String.format("9%02d%02d%d", today.monthValue, today.dayOfMonth, increment)
        val id = idStr.toLong()
        dto.setId(id)
        dto.setUserId(condition.userId)
        dto.setUserType(condition.userType)
        dto.setPlatforms(condition.platforms)
        dto.setFullUser(condition.fullUser)
        dto.setNeedResult(condition.resultNotice)
        val msgLife = condition.msgLife
        if (Objects.isNull(msgLife)) {
            dto.setMsgLifeTime(LocalDateTime.now())
        } else {
            dto.setMsgLifeTime(LocalDateTime.now().plusSeconds(msgLife.toLong()))
        }
        val element = condition.msgElement
        val msgBody = BeanTransfer.transform2MsgBody(element)
        dto.setMsgBodyList(listOf(msgBody))
        stringRedisTemplate.expireAt(uniqueCacheKey, DateUtil.toDate(today.plusDays(1L).atStartOfDay().plusHours(1L)))
        return dto
    }

    /**
     * 生成topic
     *
     * @param userId
     * @param userType
     * @param platforms
     * @param fullUser
     * @return
     */
    private fun generateTopic(userId: Long, userType: Int, platforms: List<Int?>, fullUser: Boolean): List<String?> {
        if (fullUser) {
            if (CollectionUtils.isEmpty(platforms)) {
                return listOf(Constant.FULL_TOPIC)
            }
            return platforms.stream().map { r: Int? ->
                val platformTypeEnum = BeanTransfer.transFromSdkPlatformType(r!!)
                val sysPlatform = sysPlatformService.queryPlatformByType(platformTypeEnum)
                BeanTransfer.renderTopic(Constant.PLATFORM_TOPIC, "", sysPlatform.platformCode)
            }.distinct().collect(Collectors.toList())
        }
        val accountTypeEnum = BeanTransfer.transFromSdkAccountType(userType)
        val hermes = hermesService.queryByUserId(userId, accountTypeEnum)
        BV.notNull(hermes) { "用户不存在,请先注册" }
        val commonTopic = BeanTransfer.renderTopic(Constant.USER_TOPIC, hermes.identifier, "")
        if (CollectionUtils.isEmpty(platforms)) {
            return listOf(commonTopic)
        }
        val set: MutableSet<String?> = HashSet()
        for (platform in platforms) {
            val platformTypeEnum = BeanTransfer.transFromSdkPlatformType(platform!!)
            val sysPlatform = sysPlatformService.queryPlatformByType(platformTypeEnum)
            if (Objects.nonNull(sysPlatform)) {
                set.add(BeanTransfer.renderTopic(Constant.PLATFORM_USER_TOPIC, hermes.identifier, sysPlatform.platformCode))
            }
        }
        if (CollectionUtils.isEmpty(set)) {
            return listOf(commonTopic)
        }
        return ArrayList(set)
    }
}
