package cn.fw.hermes.service.biz

import cn.fw.common.businessvalidator.Validator.BV
import cn.fw.common.util.logInfo
import cn.fw.hermes.common.constant.Constant
import cn.fw.hermes.domain.db.Hermes
import cn.fw.hermes.domain.enums.AccountTypeEnum
import cn.fw.hermes.service.data.HermesService
import com.baomidou.mybatisplus.core.toolkit.Wrappers
import com.tencent.xinge.XingeApp
import com.tencent.xinge.bean.AudienceType
import com.tencent.xinge.bean.Message
import com.tencent.xinge.bean.MessageAndroid
import com.tencent.xinge.bean.MessageType
import com.tencent.xinge.device.account.AccountBindOperatorType
import com.tencent.xinge.device.account.AccountBindRequest
import com.tencent.xinge.device.account.AccountTypePair
import com.tencent.xinge.device.account.TokenAccountsPair
import com.tencent.xinge.push.app.PushAppRequest
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.springframework.beans.factory.annotation.Value
import org.springframework.cloud.context.config.annotation.RefreshScope
import org.springframework.data.redis.core.StringRedisTemplate
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional


/**
 * 信鸽推送业务处理服务
 *
 * @author : kurisu
 * @version : 1.0
 * @desc : 信鸽推送业务处理服务
 * @date : 2024-01-04 15:54
 */
@Service
@RefreshScope
class XgPushService(
    @Value("\${spring.cache.custom.global-prefix}:xg:token")
    private val keyPrefix: String,
    private val xingeApp: XingeApp,
    private val hermesService: HermesService,
    private val stringRedisTemplate: StringRedisTemplate
) {

    /**
     * 储存和更新用户腾讯信鸽设备的token
     */
    @Transactional(rollbackFor = [Exception::class])
    fun saveUserToken(userId: Long?, accountType: AccountTypeEnum, tk: String) {
        BV.notNull(userId) { "用户[$userId]信息不存在" }
        userId?.let {
            val hermes = hermesService.queryByUserId(it, accountType)
            hermes?.run {
                val cacheKey = "${keyPrefix}:${accountType.name}"
                val hashOps = stringRedisTemplate.boundHashOps<String, String>(cacheKey)
                if (xgToken == tk) {
                    hashOps.put(it.toString(), xgToken)
                    return
                }
                val clearUserIds = blankingToken(it, accountType, tk)
                CoroutineScope(Dispatchers.IO).launch {
                    clearCoverTokenAccount(tk, identifier, false)
                }
                val her = Hermes()
                her.id = id
                her.xgToken = tk
                if (her.updateById()) {
                    hashOps.put(it.toString(), tk)
                }
                if (clearUserIds.isNotEmpty()) {
                    val clearUserList = clearUserIds.map { userId -> userId.toString() }.toTypedArray()
                    hashOps.delete(*clearUserList)
                }
            }
        }
    }

    /**
     * 解绑账号token
     */
    @Transactional(rollbackFor = [Exception::class])
    fun delUserToken(userId: Long?, accountType: AccountTypeEnum) {
        BV.notNull(userId) { "用户[$userId]信息不存在" }
        userId?.let {
            val hermes = hermesService.queryByUserId(it, accountType)
            hermes?.run {
                if (xgToken == Constant.INVALID_TOKEN) {
                    return
                }
                val cacheKey = "${keyPrefix}:${accountType.name}"
                val hashOps = stringRedisTemplate.boundHashOps<String, String>(cacheKey)
                CoroutineScope(Dispatchers.IO).launch {
                    clearCoverTokenAccount(xgToken, identifier, true)
                }
                val her = Hermes()
                her.id = id
                her.xgToken = Constant.INVALID_TOKEN
                if (her.updateById()) {
                    hashOps.delete(it.toString())
                }
            }
        }
    }

    fun sendMsg(userId: Long, accountType: AccountTypeEnum): Triple<Int, String, String> {
        val xgToken = getUserToken(userId, accountType)
        if (Constant.INVALID_TOKEN == xgToken) {
            return Triple(-1, xgToken, "INVALID_TOKEN")
        }
        val pushAppRequest = PushAppRequest()
        pushAppRequest.audience_type = AudienceType.token
        pushAppRequest.message_type = MessageType.notify
        pushAppRequest.isMulti_pkg = true
        pushAppRequest.expire_time = 60 * 5

        val message = Message()
        message.title = "\uD83D\uDD14待办提醒"
        message.content = "收到一条新的待办消息，请点击查看"
        message.showType = 1
        // 推送给安卓设备 [fixme 暂时不支持推送给ios]
        val messageAndroid = MessageAndroid()
        messageAndroid.setnChId("backlog_message")
        messageAndroid.setnCategory("CATEGORY_REMINDER")
        messageAndroid.badgeType = -2
        messageAndroid.ring_raw = "xg_ring"
        messageAndroid.hwCategory = "WORK"
        messageAndroid.oppoChId = "oppo_ch_id"
        messageAndroid.vivoCategory = "TODO"
        message.android = messageAndroid

        pushAppRequest.message = message
        pushAppRequest.token_list = arrayListOf(xgToken)
        val pushRes = this.xingeApp.pushApp(pushAppRequest)
        logInfo("给账号[{}]推送消息, 结果: {}", userId, pushRes)
        val code = pushRes.getInt("ret_code")
        val third = if (code == 0) pushRes.getString("push_id") else pushRes.getString("err_msg")
        return Triple(code, xgToken, third)
    }

    private fun getUserToken(userId: Long, accountType: AccountTypeEnum): String {
        val cacheKey = "${keyPrefix}:${accountType.name}"
        val hashOps = stringRedisTemplate.boundHashOps<String, String>(cacheKey)
        return hashOps.get(userId.toString()) ?: hermesService.queryByUserId(userId, accountType)?.xgToken ?: Constant.INVALID_TOKEN
    }

    private fun clearCoverTokenAccount(token: String, account: String, noCover: Boolean) {
        val request = AccountBindRequest()
        request.operatorType = if (noCover) AccountBindOperatorType.AllDeleteTokenFromAccount else AccountBindOperatorType.BatchCoverAccount
        val accountTypePair = AccountTypePair()
        accountTypePair.account = account

        val accountTypePairs = arrayListOf(accountTypePair)
        request.accountList = accountTypePairs

        if (!noCover) {
            val tokenAccountsPair = TokenAccountsPair()
            tokenAccountsPair.accountList = accountTypePairs
            tokenAccountsPair.token = token
            request.tokenAccounts = arrayListOf(tokenAccountsPair)
        }
        val bindRes = this.xingeApp.accountBind(request)
        logInfo("给[{}]绑定账号[{}]({}), 结果: {}", token, account, !noCover, bindRes)
    }

    private fun blankingToken(userId: Long, accountType: AccountTypeEnum, xgToken: String): List<Long> {
        val list = hermesService.list(
            Wrappers.lambdaQuery<Hermes>()
                .eq(Hermes::getXgToken, xgToken)
                .eq(Hermes::getAccountType, accountType)
                .ne(Hermes::getUserId, userId)
        )
        if (list.isNullOrEmpty()) {
            return listOf()
        }
        list.forEach { it.xgToken = Constant.INVALID_TOKEN }
        hermesService.updateBatchById(list)
        return list.map(Hermes::getUserId)
    }
}
