package cn.fw.hermes.service.emqx

import cn.fw.common.util.logError
import cn.fw.common.util.logInfo
import cn.fw.hermes.common.constant.Constant
import cn.fw.hermes.common.utils.RobotHelper
import cn.fw.hermes.common.utils.ThreadPoolUtil
import cn.fw.hermes.domain.mqtt.Message
import cn.fw.hermes.domain.mqtt.UpdateMessage
import cn.fw.hermes.service.data.MessageHistoryService
import cn.fw.hermes.service.property.MqttProperty
import cn.hutool.json.JSONUtil
import org.apache.commons.lang3.tuple.MutablePair
import org.apache.commons.lang3.tuple.Pair
import org.eclipse.paho.client.mqttv3.*
import org.springframework.stereotype.Service
import java.nio.charset.StandardCharsets
import java.util.*
import java.util.concurrent.CompletableFuture
import java.util.concurrent.atomic.AtomicBoolean

/**
 * mqtt服务
 *
 * @author : kurisu
 * @version : 2.0
 * @desc : mqtt服务
 * @date : 2023-07-20 17:46
 */
@Service
class MqttService(
    private val mqttProperty: MqttProperty,
    private val messageHistoryService: MessageHistoryService,
    private val mqttAsyncClient: MqttAsyncClient
) : MqttCallbackExtended {
    private val reconnecting = AtomicBoolean(false)
    private val TAG = "MQTT"

    fun initialization() {
        try {
            val options = MqttConnectOptions()
            options.userName = mqttProperty.userName ?: ""
            options.connectionTimeout = mqttProperty.timeout
            options.keepAliveInterval = mqttProperty.keepalive
            options.isCleanSession = true
            options.maxInflight = 1000
            mqttAsyncClient.setCallback(this)
            mqttAsyncClient.connect(options)
        } catch (me: MqttException) {
            logError("mqtt connect failed", me)
        }
    }

    /**
     * 手动重连
     *
     * @return
     */
    fun manualReConnect(): Boolean {
        if (mqttAsyncClient.isConnected) {
            return true
        }
        try {
            mqttAsyncClient.reconnect()
            return true
        } catch (e: MqttException) {
            logInfo("[{}]手动重连失败", TAG, e)
        }
        return false
    }

    fun disconnect() {
        try {
            if (mqttAsyncClient.isConnected) {
                mqttAsyncClient.disconnect()
                mqttAsyncClient.close()
            }
        } catch (e: MqttException) {
            logError("mqtt disconnect failed", e)
        }
    }

    /**
     * 发送系统更新消息
     *
     * @param topic 主题
     * @param msg   消息内容
     */
    fun senUpdateMsg(topic: String?, msg: UpdateMessage): Pair<Boolean, String>? {
        try {
            if (mqttAsyncClient.isConnected) {
                val mqttMessage = MqttMessage()
                mqttMessage.payload = JSONUtil.toJsonStr(msg).toByteArray(StandardCharsets.UTF_8)
                mqttAsyncClient.publish(topic, mqttMessage, null, null)
                return Pair.of(true, "")
            }
            return null
        } catch (me: MqttException) {
            logError("发送应用更新消息versionName: [{}] 失败：", msg.versionName, me)
            return Pair.of(false, me.message)
        }
    }

    fun senMsg(topic: String?, msg: Message): Pair<Boolean, String>? {
        val msgId = msg.msgId
        try {
            if (mqttAsyncClient.isConnected) {
                val mqttMessage = MqttMessage()
                mqttMessage.id = msgId.toInt()
                mqttMessage.payload = JSONUtil.toJsonStr(msg).toByteArray(StandardCharsets.UTF_8)
                mqttAsyncClient.publish(topic, mqttMessage, msgId, null)
                return Pair.of(true, "")
            }
            return null
        } catch (me: MqttException) {
            logError("发送消息msgId: [{}]失败：", msgId, me)
            return Pair.of(false, me.message)
        }
    }

    fun senMsg(topics: List<String?>, msg: Message): Pair<Boolean, String>? {
        val msgId = msg.msgId
        val pair = MutablePair.of(false, "发送中...")
        try {
            val connected = mqttAsyncClient.isConnected
            if (connected) {
                for (topic in topics) {
                    val mqttMessage = MqttMessage()
                    mqttMessage.id = msgId.toInt()
                    mqttMessage.qos = 1
                    mqttMessage.payload = JSONUtil.toJsonStr(msg).toByteArray(StandardCharsets.UTF_8)
                    mqttAsyncClient.publish(topic, mqttMessage, msgId, null)
                    if (pair.key != true) {
                        pair.setLeft(true)
                    }
                }
                return pair
            }
            logError("client is not connected")
            return null
        } catch (me: Exception) {
            logError("发送消息msgId: [{}]失败：", msgId, me)
            if (pair.key != true) {
                pair.setLeft(false)
                pair.setRight(me.message)
            }
        }
        return pair
    }

    /**
     * 连接丢失
     *
     * @param cause the reason behind the loss of connection.
     */
    override fun connectionLost(cause: Throwable) {
        val compared = reconnecting.compareAndSet(false, true)
        if (!compared) {
            return
        }
        logInfo("[{}]断开连接：{}", TAG, cause.message)
        for (i in 0..9) {
            try {
                if (mqttAsyncClient.isConnected) {
                    break
                }
                logInfo("[{}]重新连接中({})...", TAG, i)
                mqttAsyncClient.reconnect()
                Thread.sleep(128L shl i)
            } catch (e: MqttException) {
                logInfo("[{}]重接失败", TAG, e)
            } catch (ignored: InterruptedException) {
            }
        }
        if (!mqttAsyncClient.isConnected) {
            reconnecting.set(false)
            logInfo("[{}]重连次数耗尽", TAG)
            sendWarningMsg()
        }
    }

    /**
     * 收到消息
     *
     * @param topic   name of the topic on the message was published to
     * @param message the actual message.
     */
    override fun messageArrived(topic: String, message: MqttMessage) {
        logInfo("{}, 接收消息主题: [{}]-接收消息内容: [{}]", TAG, topic, String(message.payload, StandardCharsets.UTF_8))
    }

    /**
     * 连接成功
     *
     * @param reconnect 是否是重连 If true, the connection was the result of automatic reconnect.
     * @param serverURI The server URI that the connection was made to.
     */
    override fun connectComplete(reconnect: Boolean, serverURI: String) {
        reconnecting.set(false)
        try {
            mqttAsyncClient.subscribe(Constant.SYS_TOPIC, 1)
        } catch (e: MqttException) {
            logError("subscribe topic [{}] failed", Constant.SYS_TOPIC, e)
        }
    }

    /**
     * 完成消息推送完成后的回调
     *
     * @param token the delivery token associated with the message.
     */
    override fun deliveryComplete(token: IMqttDeliveryToken) {
        val userContext = token.userContext
        if (userContext != null) {
            val id = userContext as Long
            logInfo("完成消息id[{}]的推送", id)
            CompletableFuture.runAsync({
                try {
                    // 1.5秒后更新状态
                    Thread.sleep(1500)
                } catch (ignored: InterruptedException) {
                }
                messageHistoryService.sendCallback(id, true, Constant.SUCCESS_MSG)
            }, ThreadPoolUtil.executor)
        }
    }

    /**
     * 手动发送预警消息
     *
     * @return
     */
    fun sendWarningMsg() {
        val robot = mqttProperty.robot
        logInfo("机器人配置信息：[{}]", robot)
        if (Objects.nonNull(robot) && robot!!.enabled) {
            try {
                val res = RobotHelper.sendEarlyWarning(robot.accessToken, robot.secret)
                logInfo("发送预警消息：[{}]", res)
            } catch (e: Exception) {
                logInfo("发送预警失败", e)
            }
        } else {
            logInfo("机器人预警通知未启用：[{}]", robot)
        }
    }
}
